import java.util.Stack;
import java.util.Scanner;

/** Iterative solution for the Towers of Hanoi problem up through 6 disks
  * @author Bob Wilson
  * @version 03/14/2009
  */

public class Hanoi
{
  /** Main method to get the number of disks from the user and solve
    * @param args Not used
    */
   public static void main(String [] args)
   {
       Scanner scan = new Scanner(System.in);
       int height = 0;
       
       do {
         System.out.println("Enter Number of Disks (1 - 6):");
         if (scan.hasNextInt())
           height = scan.nextInt();
         else {
           scan.next();
           System.out.println("Error - enter an integer from 1 to 6");
         }
       } while (height < 1 || height > 6);
       
       towers(height);
   }
   
   private static void towers(int height)
   {
     // Set up the array of three pegs
     Stack<Integer> [] pegs = (Stack<Integer> []) new Stack[3];
     for (int i = 0; i < 3; i++)
       pegs[i] = new Stack<Integer>();

     // Put all the disks on the initial peg
     for (int i = height; i > 0; i--)
       pegs[0].push(new Integer(i));
     int smallest = 0;
     
     // display the initial position
     display(pegs, height);

     // Calculate (2 to the power of the height) - 1
     int TPN = (int) (Math.pow(2, height) - 0.5);
     
     for (int i = 0; i < TPN; i++)            // O(2 to the N)
     {
       if (i % 2 == 0)   
       {                                      // move the smallest disk from its peg in sequence
         int newSmallest = height % 2 == 0 ? (smallest + 1) % 3 : smallest > 0 ? smallest - 1 : 2;
         pegs[newSmallest].push(pegs[smallest].pop());
         smallest = newSmallest;
       }
       else              
       {                                     // can't move from or to the peg with smallest disk
                                             // must move between the two remaining pegs 
         Stack<Integer> pega = pegs[(smallest + 1) % 3];
         Stack<Integer> pegb = pegs[(smallest + 2) % 3];
         
         if (pega.isEmpty())
         {
           if (pegb.isEmpty())
           {                                 // neither can be the source??
             System.out.println("Error");
             System.exit(0);
           }
           else
           {                        
             pega.push(pegb.pop());          // pegb must be the source
           }
         }
         else
         {
           if (pegb.isEmpty())  
           {
             pegb.push(pega.pop());          // pega must be the source
           }
           else
           {                                 // figure out which is the source
             if(pega.peek() > pegb.peek())
               pega.push(pegb.pop());        // pegb must be the source
             else
               pegb.push(pega.pop());        // pega must be the source
           }
         }
       }
       
       // display the current position
       display(pegs, height);
     }
   }
   
   private static void display(Stack<Integer> [] origPegs, int height)
   {
     String [] disks = {"       |       ",     // blank space
                        "      *|*      ",     // smallest disk
                        "     **|**     ", 
                        "    ***|***    ", 
                        "   ****|****   ",
                        "  *****|*****  ",
                        " ******|****** "};    // largest disk
     
     Stack<Integer> [] pegs = (Stack<Integer> []) new Stack[3];
     
     for (int i = 0; i < 3; i++)                        // make copies so that we can pop
       pegs[i] = (Stack<Integer>) origPegs[i].clone();
     
     System.out.println("\n");                          // print a couple empty lines
     
     for (int i = height; i > 0; i--) {                 // print the piles of disks on each peg
       for (int j = 0; j < 3; j++)
       {
         if (i > pegs[j].size())                        // is stack on peg high enough to print?
           System.out.print(disks[0]);                  // no, print blank space
         else 
           System.out.print(disks[pegs[j].pop()]);      // yes
       }
       System.out.println();
     }
     System.out.println("*******^**************^**************^*******");
   }

   // Another iterative solution from the net
   // Reference cited:
   // http://obelix.ee.duth.gr/~apostolo/TowersOfHanoi/hanoiC.html
   
   private static void hanoi(int height)
   {
       int fromPole, toPole, Disk;
       int[]   BitStr = new int[height],
               Hold   = new int[height];
       char[]  Place  = {'A', 'C', 'B'};
       int i, j, temp;
 
       for (i=0; i < height; i++)
       {
          BitStr[i] = 0;
          Hold[i] = 1;
       }
       temp = 3 - (height % 2);
       int TotalMoves = (int) Math.pow(2, height) - 1;
       for (i=1; i <= TotalMoves; i++)
       {
           for (j=0 ; BitStr[j] != 0; j++)
              BitStr[j] = 0;
           BitStr[j] = 1;
           Disk = j+1;
           if (Disk == 1)
           {
               fromPole = Hold[0];
               toPole = 6 - fromPole - temp;
               temp = fromPole;
           }
           else
           {
              fromPole = Hold[Disk-1];
              toPole = 6 - Hold[0] - Hold[Disk-1];
           }        
           moveDisk(Place[fromPole-1], Place[toPole-1]);
           Hold[Disk-1] = toPole;
      }
   }

   private static void moveDisk(char from, char to)
   {
      System.out.println("Move one disk from: " + from + " to: " + to);
   }
}